package at.jku.fim.datalinksimulation.layer1.wire;

import at.jku.fim.datalinksimulation.layer1.data.RawData;

/**
 * <p>Title: OSI Simulation Framework</p>
 * This interface must be implemented by classes which are
 * interested in data sent on a wire.
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: WireConnector.java,v 1.1 2005/10/27 13:10:11 aputzinger Exp $
 */

public interface WireConnector {

    /**
     * This method is called whenever a data package is received by a subscribed wire.
     * @param sender If the sender is a Wire, sender contains the reference to the wire which received the packet.
     * @param data The received data on the wire
     */
    public void informAboutDataOnWire(Wire sender, RawData data);

    /**
     * This method should only be called by Wires, which inform this WireConnector that they are connected.
     * @param wire Connected wire
     * @throws WiringException May be thrown to veto against this wiring
     */
    void bePlugged(Wire wire, int portNr) throws WiringException;

    /**
     * This method should only be called by Wires, which inform this WireConnector that they are disconnected
     * @param wire Disconnected wire
     * @throws WiringException May be thrown on any error or veto
     */
    void beUnplugged(Wire wire, int portNr) throws WiringException;

}