package at.jku.fim.datalinksimulation.layer1.wire.damagedwire;

import at.jku.fim.datalinksimulation.layer1.data.RawData;
import at.jku.fim.datalinksimulation.layer1.wire.StandardWire;
import at.jku.fim.datalinksimulation.layer1.wire.WireConnector;
import at.jku.fim.datalinksimulation.utils.Logger;

/**
 * A breakable wire: Works perfectly or doesn't pass on anything at all.
 * <p>Copyright: (c) 2004</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Michael Sonntag
 * @version $Id$
 */
public class BreakableWire extends StandardWire {

    /** Logger; otherwise it won't be shown in the log window */
    protected Logger logger = new Logger("Breakable Wire");

  /** If value is set to true, the wire is broken and blocks completely*/
    protected boolean broken=false;

    /**
     * Constructor
     */
    public BreakableWire() {
        super();
    }

    /*
     *  Either sends all data on perfectly or, if broken, doesn't send anything.
     * @see at.jku.fim.datalinksimulation.layer1.wire.Wire#send(at.jku.fim.datalinksimulation.data.RawData, java.util.Vector)
     */
    public void send(WireConnector sender, RawData data) {
        if(!broken) {
            super.send(sender,data);
        }
    }

    /** Retrieve the wire state
     * @return whether the wire is broken or not 
     */
    public boolean isBroken() {
        return broken;
    }

    /**
     * Set the wire state
     * @param broken if <code>true</code> the wire is broken
     */
    public void setBroken(boolean broken) {
        this.broken = broken;
        logger.log("Wire is now "+(broken?"broken":"good"));
    }

}