package at.jku.fim.datalinksimulation.layer2.devices;

import at.jku.fim.datalinksimulation.layer1.devices.Hub;
import at.jku.fim.datalinksimulation.layer2.filters.EasyL2CRCCheckFilter;
import at.jku.fim.datalinksimulation.layer2.filters.EasyL2v2DestuffingFilter;
import at.jku.fim.datalinksimulation.layer2.filters.EasyL2v2StuffingFilter;
import at.jku.fim.datalinksimulation.networkstack.incoming.DecodingNetworkFilter;
import at.jku.fim.datalinksimulation.networkstack.incoming.IncomingNetworkStackConfiguration;
import at.jku.fim.datalinksimulation.networkstack.outgoing.EncodingNetworkFilter;
import at.jku.fim.datalinksimulation.networkstack.outgoing.OutgoingNetworkStackConfiguration;

/**
 * <p>Title: OSI Simulation Framework</p>
 * This class represents a version of an "intelligent hub", which checks,
 * if frames on layer 2 are corrupt or not; in case of an invalid frame 
 * the frame is simply dropped and not forwarded. Because this device
 * knows things about facts in layer 2, but it doesn't do switching,
 * it can be regarded as a "switch in flooding mode".
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: CRCGuard.java,v 1.1 2005/10/27 13:10:11 aputzinger Exp $
 */

public class CRCGuard extends Hub {

	/**
	 * Constructor
	 * @param identifier Identifier for logs
	 * @param numberOfPorts Number of available ports
	 */
	public CRCGuard(String identifier, int numberOfPorts) {
        super(identifier, numberOfPorts);
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.genericdevices.InterlinkingDevice#getIncomingNetworkStackConfiguration()
	 */
	protected IncomingNetworkStackConfiguration getIncomingNetworkStackConfiguration() {
		return new IncomingNetworkStackConfiguration() {
		    DecodingNetworkFilter[][] filters = new DecodingNetworkFilter[6][];

		    {   filters[1] = new DecodingNetworkFilter[2];
		        filters[1][0] = new EasyL2v2DestuffingFilter();
		        filters[1][1] = new EasyL2CRCCheckFilter();		        
		    }

		    /*
			 * (non-Javadoc)
			 * 
			 * @see at.jku.fim.datalinksimulation.networkstack.IncomingNetworkStackConfiguration#getDecodingFilters(int)
			 */
		    public DecodingNetworkFilter[] getDecodingFilters(int osiLevel) {
		        if (osiLevel >= 1 && osiLevel <= 6) {
		            return filters[osiLevel - 1];
		        }
		        return null;
		    }
		};
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.genericdevices.InterlinkingDevice#getOutgoingNetworkStackConfiguration()
	 */
	protected OutgoingNetworkStackConfiguration getOutgoingNetworkStackConfiguration() {
		return new OutgoingNetworkStackConfiguration() {

		    EncodingNetworkFilter[][] filters = new EncodingNetworkFilter[6][];

		    {   filters[1] = new EncodingNetworkFilter[1];
		        filters[1][0] = new EasyL2v2StuffingFilter();
		    }
			
			public EncodingNetworkFilter[] getEncodingFilters(int osiLevel) {
		        if (osiLevel >= 1 && osiLevel <= 6) {
		            return filters[osiLevel - 1];
		        }
		        return null;
			}
			
		};
	}
	
}