package at.jku.fim.datalinksimulation.layer2.filters;

import java.util.BitSet;

import at.jku.fim.datalinksimulation.layer1.data.RawData;
import at.jku.fim.datalinksimulation.layer2.protocols.EasyL2v2Utils;
import at.jku.fim.datalinksimulation.networkstack.NetworkStackParameters;
import at.jku.fim.datalinksimulation.networkstack.incoming.DecodingNetworkFilter;
import at.jku.fim.datalinksimulation.utils.ConversionUtils;
import at.jku.fim.datalinksimulation.utils.Logger;

/**
 * <p>Title: OSI Simulation Framework</p>
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: EasyL2v2DestuffingFilter.java,v 1.1 2005/10/27 13:10:07 aputzinger Exp $
 */

public class EasyL2v2DestuffingFilter implements DecodingNetworkFilter {

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.networkstack.NetworkFilter#getFilterOSILayer()
	 */
    public int[] getFilterOSILayer() {
        int[] result = new int[1];
        result[0] = OSI_LAYER_2;
        return result;
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.networkstack.DecodingNetworkFilter#decode(at.jku.fim.datalinksimulation.networkstack.NetworkStackParameters,
	 *      at.jku.fim.datalinksimulation.data.RawData,
	 *      at.jku.fim.datalinksimulation.utils.Logger)
	 */
    public RawData decode(NetworkStackParameters params, RawData data,
            Logger logger) {

    	BitSet bs = ConversionUtils.createBitSet(data.getRawDataBulk());
    	logger.log("Frame before decoding: " + ConversionUtils.getStringRepresentation(bs));

    	bs = EasyL2v2Utils.convert_EasyL2v2_To_EasyL2(bs, logger);
    	
    	if (bs != null) {
        	logger.log("Frame after decoding: " + ConversionUtils.getStringRepresentation(bs));
        	
    		RawData result = new RawData(ConversionUtils.createFrame(bs, (bs.length()/8) + ((bs.length() % 8 == 0) ? 0 : 1)));
    		return result;
    	}
    	return null;
    }

    /*
     *  (non-Javadoc)
     * @see at.jku.fim.datalinksimulation.networkstack.DecodingNetworkFilter#getDescription()
     */
    public String getDescription() {
        return "EasyL2v2 frame -> EasyL2 frame (remove frame markers + bit stuffing)";
    }

}