package at.jku.fim.datalinksimulation.layer2.filters;

import java.util.BitSet;

import at.jku.fim.datalinksimulation.layer1.data.RawData;
import at.jku.fim.datalinksimulation.layer2.protocols.EasyL2v2Utils;
import at.jku.fim.datalinksimulation.networkstack.NetworkStackParameters;
import at.jku.fim.datalinksimulation.networkstack.outgoing.EncodingNetworkFilter;
import at.jku.fim.datalinksimulation.utils.ConversionUtils;
import at.jku.fim.datalinksimulation.utils.Logger;

/**
 * <p>Title: OSI Simulation Framework</p>
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: EasyL2v2StuffingFilter.java,v 1.1 2005/10/27 13:10:07 aputzinger Exp $
 */

public class EasyL2v2StuffingFilter implements EncodingNetworkFilter {

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.networkstack.EncodingNetworkFilter#encode(at.jku.fim.datalinksimulation.networkstack.NetworkStackParameters,
	 *      at.jku.fim.datalinksimulation.data.RawData,
	 *      at.jku.fim.datalinksimulation.utils.Logger)
	 */
    public RawData encode(NetworkStackParameters params, RawData data,
            Logger logger) {

    	BitSet bs = ConversionUtils.createBitSet(data.getRawDataBulk());

    	logger.log("Frame before encoding: " + ConversionUtils.getStringRepresentation(bs));
    	
    	bs = EasyL2v2Utils.convert_EasyL2_To_EasyL2v2(bs, logger);
    	
    	if (bs != null) {
        	logger.log("Frame after encoding: " + ConversionUtils.getStringRepresentation(bs));    		
    		
    		RawData result = new RawData(ConversionUtils.createFrame(bs, (bs.length()/8) + ((bs.length() % 8 == 0) ? 0 : 1)));
    		return result;
    	}
    	return null;
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.networkstack.NetworkFilter#getFilterOSILayer()
	 */
    public int[] getFilterOSILayer() {
        int[] layers = new int[1];
        layers[0] = OSI_LAYER_2;
        return layers;
    }

    /*
	 * (non-Javadoc)
	 * 
	 * @see at.jku.fim.datalinksimulation.networkstack.EncodingNetworkFilter#getDescription()
	 */
    public String getDescription() {
        return "EasyL2 frame -> EasyL2v2 frame (added frame markers + bit stuffing)";
    }

}