package at.jku.fim.datalinksimulation.utils;

import java.util.BitSet;

/**
 * <p>Title: OSI Simulation Framework</p>
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: ConversionUtils.java,v 1.1 2005/10/27 13:10:11 aputzinger Exp $
 */

public class ConversionUtils {

    // speed up hex conversion by creating lookup table
    private final static String hexTable = "0123456789ABCDEF";

    // speed up pow calc
	private final static int[] pows = {1,2,4,8,16,32,64,128};
    
    /**
     * Converts a given String to a sequence of hexadecimal expressions; only works fine
     * for characters between 0 and 255
     * @param data String to convert; if data == null, an empty String is returned
     * @return Hexadecimal representation of String
     */ 
    public static String stringToHex(String data) {
        if (data == null)
            return "";
        StringBuffer result = new StringBuffer();
        for (int i = 0; i < data.length(); i++) {
            int charIntValue = data.charAt(i);
            result.append(byteToHex(charIntValue));
            if (charIntValue > 31 && charIntValue < 256) {
                result.append("(").append(data.charAt(i)).append(")");
            }
            if ((i+1) < data.length())
                result.append(" ");
        }
        return result.toString();
    }

    /**
     * Converts a value between 0 and 255 to it's hex representation
     * @param data value to convert
     * @return hex representation
     */
    public static String byteToHex(int data) {
        if (data < 0 || data > 255)
            data = 0;
        int pos1 = (data / 16);
        int pos2 = (data % 16);
        return ("" + hexTable.charAt(pos1) + hexTable.charAt(pos2));
    }

	public static BitSet createBitSet(int[] rd) {
		BitSet bs = new BitSet(rd.length * 8);
		
		for (int bytes = 0; bytes < rd.length; bytes++) {
			for (int bits = 0; bits < 8; bits++) {
				if ((rd[bytes] & pows[bits]) == pows[bits]) {
					bs.set((rd.length-1-bytes)*8 + bits);
				}
			}
		}
		
		return bs;
	}

	public static int[] createFrame(BitSet bs, int arraySize) {
		int[] frame = new int[arraySize];
		
		for (int bits = 0; bits < bs.length(); bits++) {
			if (bs.get(bits)) {
				frame[frame.length - 1 - (bits/8)] += pows[bits % 8];
			}
		}
		
		return frame;
	}

	public static String getStringRepresentation(BitSet bs) {
		StringBuffer buffer = new StringBuffer("");
		for (int i = 0; i < bs.length(); i++) {
			if (i > 0 && (i % 8) == 0)
				buffer.append(" | ");
			buffer.append(bs.get(i) ? "1" : "0");
		}
		return buffer.reverse().toString();
	}
	
}