package at.jku.fim.datalinksimulation.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * <p>Title: OSI Simulation Framework</p>
 * <p>Copyright: (c) 2004-05</p>
 * <p>Company: FIM@JKU, www.fim.uni-linz.ac.at</p>
 * @author Andreas Putzinger
 * @version $Id: Logger.java,v 1.1 2005/10/27 13:10:11 aputzinger Exp $
 */

public class Logger {

	/* +++++++++++++++++++++++++++++++++++++
	 * + STATIC PART
	 * +++++++++++++++++++++++++++++++++++++ */
	
	/**
	 * Root loggers
	 */
    private static Vector rootLoggers = new Vector();

    /**
     * Global logging time stamp
     */
    public static int globalTime = 0;

    /**
     * Empties all logs
     */
    public static void clearAllLogs() {
        Iterator it = rootLoggers.iterator();
        while (it.hasNext()) {
            Logger logger = (Logger) it.next();
            logger.clear();
        }
    }

    /**
     * @return Returns a (copied) list of root loggers
     */
    public static List getRootLoggers() {
    	return (List)rootLoggers.clone();
    }    

    
	/* +++++++++++++++++++++++++++++++++++++
	 * + INSTANCE PART
	 * +++++++++++++++++++++++++++++++++++++ */
    
    /** Logger description */
    protected String description;

    /** logged text */
    protected StringBuffer logs = new StringBuffer();
    
    /** last time when an event was logged */
    protected int lastTimeStamp = -1;
    
    /** children loggers */
    protected ArrayList childrenLoggers = new ArrayList();

    /** parent logger */
    protected Logger parentLogger = null;
    
    /**
     * Constructor to create a new logger for a component 
     * @param description Description/name of logger component
     */
    public Logger(String description) {
        this.description = description;
        rootLoggers.add(this);
    }

    /**
     * Log an event/message
     * @param msg The event/message to log
     */
    public void log(String msg) {
        if (lastTimeStamp != globalTime)
            logs.append("-----------------------------------------------------------------\n");
        msg = "" + globalTime + " --> " + msg + "\n";
        logs.append(msg);
        lastTimeStamp = globalTime;
    }

    /**
     * @return Returns the hierarchy level of this logger
     */
    private int getDepth() {
    	if (parentLogger == null)
    		return 1;
   		return 1 + parentLogger.getDepth();
    }
    
    /**
     * Each logger is represented with his description
     */
    public String toString() {
    	StringBuffer prefix = new StringBuffer("");
    	for (int i = 0; i < getDepth()-1; i++)
    		prefix.append("     ");
        return prefix.toString() + description;
    }

    /**
     * @return Returns logged events
     */
    public String getLog() {
        return logs.toString();
    }

    /**
     * Clears all logged events
     */
    public void clear() {
        logs = new StringBuffer();
        for (Iterator it = childrenLoggers.iterator(); it.hasNext(); ) {
        	((Logger)it.next()).clear();
        }
    }
    
    private void setParentLogger(Logger parent) {
    	parentLogger = parent;
    }
    
    /**
     * Define a logger as logging child
     * @param child The child
     */
    public void addChildLogger(Logger child) {
    	if (child != null) {
    		childrenLoggers.add(child);
    		rootLoggers.remove(child);
    		child.setParentLogger(this);
    	}
    }
    
    /**
     * @return Return a list of all children loggers
     */
    public List getChildrenLoggers() {
    	return (List)childrenLoggers.clone();
    }
}